<?php
/*****************************************************************************
	PHP/GD Antialiasing, Proof of Concept
	Zac Hester - 2011-06-28

	Just a small bit of code to see if it can be done, and what kind of
	memory overhead is needed to pull off an arbitrary, geometry-agnostic
	antialiasing that works with transparent images.
*****************************************************************************/

//Pretty-printing for memory sizes
function get_size($nbytes) {
	$i = 0; $units = array('B', 'kB', 'MB', 'GB', 'TB');
	while($nbytes >= 1024) { $nbytes /= 1024; ++$i; }
	return sprintf('%01.2f %s', $nbytes, $units[$i]);
}

//A few configurable (within limits) values
$tw = isset($_GET['w']) && ($_GET['w'] <= 640) ? intval($_GET['w'])   : 320;
$th = isset($_GET['h']) && ($_GET['h'] <= 640) ? intval($_GET['h'])   : 240;
$aa = isset($_GET['a']) && ($_GET['a'] <= 4)   ? intval($_GET['a'])   : 4;
$wf = isset($_GET['f']) && ($_GET['f'] <= 4 )  ? floatval($_GET['f']) : 0.75;

//The AA source is an over-sampled image
$aw = $tw * $aa;
$ah = $th * $aa;

$usage = array(memory_get_usage());

//Create the over-sampled image, and the final image
$im0 = imagecreatetruecolor($aw, $ah);
$im1 = imagecreatetruecolor($tw, $th);

//Configure both images to use alpha channels
imagealphablending($im0, false);
imagesavealpha($im0, true);
imagealphablending($im1, false);
imagesavealpha($im1, true);

//Set up colors and transparency
$color0 = imagecolorallocate($im0, 0, 0, 255);
$color1 = imagecolorallocate($im1, 0, 0, 255);
$trans0 = imagecolorallocatealpha($im0, 255, 255, 255, 127);
$trans1 = imagecolorallocatealpha($im1, 255, 255, 255, 127);

//Override the default backgrounds in both images
imagefilledrectangle($im0, 0, 0, $aw, $ah, $trans0);
imagefilledrectangle($im1, 0, 0, $tw, $th, $trans1);

//AA graphics
//Note: the weighting factor can be used to provide "heavier" and "lighter"
//  subpixel interpolation in the final image.
imagesetthickness($im0, ($aa*$wf));
//Note: imagearc() interprets a 360deg arc as an ellipse, and imageellipse()
//  has problems with imagesetthickness().
imagearc($im0, ($aw/4), ($ah/2), ($aw/3), ($aw/3), 0, 359.9, $color0);
imageline($im0, ($aw/8), ($ah/8), ($aw*3/8), ($ah*7/8), $color0);
imageline($im0, ($aw/8), ($ah*7/8), ($aw*3/8), ($ah/8), $color0);
imageline($im0, ($aw/8), ($ah/2), ($aw*3/8), ($ah/2), $color0);
imageline($im0, ($aw/4), ($ah/8), ($aw/4), ($ah*7/8), $color0);

//Downsize and interpolate to provide AA
imagecopyresampled($im1, $im0, 0, 0, 0, 0, $tw, $th, $aw, $ah);

$usage[] = memory_get_usage();

imagedestroy($im0);

//Non-AA graphics (for comparison)
imagearc($im1, ($tw*3/4), ($th/2), ($tw/3), ($tw/3), 0, 359.9, $color1);
imageline($im1, ($tw*5/8), ($th/8), ($tw*7/8), ($th*7/8), $color1);
imageline($im1, ($tw*5/8), ($th*7/8), ($tw*7/8), ($th/8), $color1);
imageline($im1, ($tw*5/8), ($th/2), ($tw*7/8), ($th/2), $color1);
imageline($im1, ($tw*3/4), ($th/8), ($tw*3/4), ($th*7/8), $color1);

$usage[] = memory_get_usage();

//Print memory usage information on the image
$str = implode(', ', array_map('get_size', $usage));
$fx = ($tw/2) - ((imagefontwidth(3) * strlen($str)) / 2);
imagestring($im1, 3, $fx, ($th-20), $str, $color1);

//Output the image
header('Content-Type: image/png');
imagepng($im1);
imagedestroy($im1);

?>
